@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: correct.c,v 1.52 1994/05/25 04:29:24 geoff Exp $";
+    "$Id: correct.c,v 1.58 1994/11/02 06:56:00 geoff Exp $";
 #endif
 
 /*
@@ -50,6 +50,30 @@ static char Rcs_Id[] =
 
 /*
  * $Log: correct.c,v $
+ * Revision 1.58  1994/11/02  06:56:00  geoff
+ * Remove the anyword feature, which I've decided is a bad idea.
+ *
+ * Revision 1.57  1994/10/26  05:12:39  geoff
+ * Try boundary characters when inserting or substituting letters, except
+ * (naturally) at word boundaries.
+ *
+ * Revision 1.56  1994/10/25  05:46:30  geoff
+ * Fix an assignment inside a conditional that could generate spurious
+ * warnings (as well as being bad style).  Add support for the FF_ANYWORD
+ * option.
+ *
+ * Revision 1.55  1994/09/16  04:48:24  geoff
+ * Don't pass newlines from the input to various other routines, and
+ * don't assume that those routines leave the input unchanged.
+ *
+ * Revision 1.54  1994/09/01  06:06:41  geoff
+ * Change erasechar/killchar to uerasechar/ukillchar to avoid
+ * shared-library problems on HP systems.
+ *
+ * Revision 1.53  1994/08/31  05:58:38  geoff
+ * Add code to handle extremely long lines in -a mode without splitting
+ * words or reporting incorrect offsets.
+ *
  * Revision 1.52  1994/05/25  04:29:24  geoff
  * Fix a bug that caused line widths to be calculated incorrectly when
  * displaying lines containing tabs.  Fix a couple of places where
@@ -667,7 +691,7 @@ static void inserttoken (buf, start, tok, curchar)
 	while (*tok)
 	    *p++ = *tok++;
 	}
-    while (*p++ = *q++)
+    while ((*p++ = *q++) != '\0')
 	;
     }
 
@@ -842,6 +866,8 @@ static void wrongletter (word)
 	    {
 	    if (Try[j] == savechar)
 		continue;
+	    else if (isboundarych (Try[j])  &&  (i == 0  ||  i == n - 1))
+		continue;
 	    newword[i] = Try[j];
 	    if (good (newword, 0, 1, 0, 0))
 		{
@@ -888,6 +914,8 @@ static void missingletter (word)
 	{
 	for (i = 0;  i < Trynum;  i++)
 	    {
+	    if (isboundarych (Try[i])  &&  r == newword)
+		continue;
 	    *r = Try[i];
 	    if (good (newword, 0, 1, 0, 0))
 		{
@@ -899,6 +927,8 @@ static void missingletter (word)
 	}
     for (i = 0;  i < Trynum;  i++)
 	{
+	if (isboundarych (Try[i]))
+	    continue;
 	*r = Try[i];
 	if (good (newword, 0, 1, 0, 0))
 	    {
@@ -1378,7 +1408,7 @@ static char * getline (s)
 	    *p = 0;
 	    return (s);
 	    }
-	else if (c == erasechar)
+	else if (c == uerasechar)
 	    {
 	    if (p != s)
 		{
@@ -1388,7 +1418,7 @@ static char * getline (s)
 		backup ();
 		}
 	    }
-	else if (c == killchar)
+	else if (c == ukillchar)
 	    {
 	    while (p != s)
 		{
@@ -1408,9 +1438,12 @@ static char * getline (s)
 
 void askmode ()
     {
+    int			bufsize;	/* Length of contextbufs[0] */
+    int			ch;		/* Next character read from input */
     register char *	cp1;
     register char *	cp2;
     ichar_t *		itok;		/* Ichar version of current word */
+    int			hadnl;		/* NZ if \n was at end of line */
 
     if (fflag)
 	{
@@ -1423,9 +1456,55 @@ void askmode ()
 
     (void) printf ("%s\n", Version_ID[0]);
 
-    while (fflush (stdout),
-      xgets (contextbufs[0], sizeof contextbufs[0], stdin) != NULL)
+    contextoffset = 0;
+    while (1)
 	{
+	(void) fflush (stdout);
+	/*
+	 * Only read in enough characters to fill half this buffer so that any
+	 * corrections we make are not likely to cause an overflow.
+	 */
+	if (contextoffset == 0)
+	    {
+	    if (xgets (contextbufs[0], (sizeof contextbufs[0]) / 2, stdin)
+	      == NULL)
+		break;
+	    }
+	else
+	    {
+	    if (fgets (contextbufs[0], (sizeof contextbufs[0]) / 2, stdin)
+	      == NULL)
+		break;
+	    }
+	/*
+	 * If we didn't read to end-of-line, we may have ended the
+	 * buffer in the middle of a word.  So keep reading until we
+	 * see some sort of character that can't possibly be part of a
+	 * word. (or until the buffer is full, which fortunately isn't
+	 * all that likely).
+	 */
+	bufsize = strlen (contextbufs[0]);
+	if (contextbufs[0][bufsize - 1] == '\n')
+	    {
+	    hadnl = 1;
+	    contextbufs[0][--bufsize] = '\0';
+	    }
+	else
+	    hadnl = 0;
+	if (bufsize == (sizeof contextbufs[0]) / 2 - 1)
+	    {
+	    ch = (unsigned char) contextbufs[0][bufsize - 1];
+	    while (bufsize < sizeof contextbufs[0] - 1
+	      &&  (iswordch ((ichar_t) ch)  ||  isboundarych ((ichar_t) ch)
+	      ||  isstringstart (ch)))
+		{
+		ch = getc (stdin);
+		if (ch == EOF)
+		    break;
+		contextbufs[0][bufsize++] = (char) ch;
+		contextbufs[0][bufsize] = '\0';
+		}
+	    }
 	/*
 	** *line is like `i', @line is like `a', &line is like 'u'
 	** `#' is like `Q' (writes personal dictionary)
@@ -1434,57 +1513,67 @@ void askmode ()
 	** `~' followed by a filename sets parameters according to file name
 	** `^' causes rest of line to be checked after stripping 1st char
 	*/
-	if (contextbufs[0][0] == '*'  ||  contextbufs[0][0] == '@')
-	    treeinsert(ichartosstr (strtosichar (contextbufs[0] + 1, 0), 1),
-	      ICHARTOSSTR_SIZE,
-	      contextbufs[0][0] == '*');
-	else if (contextbufs[0][0] == '&')
-	    {
-	    itok = strtosichar (contextbufs[0] + 1, 0);
-	    lowcase (itok);
-	    treeinsert (ichartosstr (itok, 1), ICHARTOSSTR_SIZE, 1);
-	    }
-	else if (contextbufs[0][0] == '#')
-	    {
-	    treeoutput ();
-	    math_mode = 0;
-	    LaTeX_Mode = 'P';
-	    }
-	else if (contextbufs[0][0] == '!')
-	    terse = 1;
-	else if (contextbufs[0][0] == '%')
-	    terse = 0;
-	else if (contextbufs[0][0] == '-')
-	    {
-	    math_mode = 0;
-	    LaTeX_Mode = 'P';
-	    tflag = 0;
-	    }
-	else if (contextbufs[0][0] == '+')
-	    {
-	    math_mode = 0;
-	    LaTeX_Mode = 'P';
-	    tflag = strcmp (&contextbufs[0][1], "nroff") != 0
-	      &&  strcmp (&contextbufs[0][1], "troff") != 0;
-	    }
-	else if (contextbufs[0][0] == '~')
-	    {
-	    defdupchar = findfiletype (&contextbufs[0][1], 1, (int *) NULL);
-	    if (defdupchar < 0)
-		defdupchar = 0;
-	    }
+	if (contextoffset != 0)
+	    checkline (stdout);
 	else
 	    {
-	    if (contextbufs[0][0] == '^')
+	    if (contextbufs[0][0] == '*'  ||  contextbufs[0][0] == '@')
+		treeinsert(ichartosstr (strtosichar (contextbufs[0] + 1, 0), 1),
+		  ICHARTOSSTR_SIZE,
+		  contextbufs[0][0] == '*');
+	    else if (contextbufs[0][0] == '&')
 		{
-		/* Strip off leading uparrow */
-		for (cp1 = contextbufs[0], cp2 = contextbufs[0] + 1;
-		  (*cp1++ = *cp2++) != '\0';
-		  )
-		    ;
+		itok = strtosichar (contextbufs[0] + 1, 0);
+		lowcase (itok);
+		treeinsert (ichartosstr (itok, 1), ICHARTOSSTR_SIZE, 1);
+		}
+	    else if (contextbufs[0][0] == '#')
+		{
+		treeoutput ();
+		math_mode = 0;
+		LaTeX_Mode = 'P';
+		}
+	    else if (contextbufs[0][0] == '!')
+		terse = 1;
+	    else if (contextbufs[0][0] == '%')
+		terse = 0;
+	    else if (contextbufs[0][0] == '-')
+		{
+		math_mode = 0;
+		LaTeX_Mode = 'P';
+		tflag = 0;
+		}
+	    else if (contextbufs[0][0] == '+')
+		{
+		math_mode = 0;
+		LaTeX_Mode = 'P';
+		tflag = strcmp (&contextbufs[0][1], "nroff") != 0
+		  &&  strcmp (&contextbufs[0][1], "troff") != 0;
+		}
+	    else if (contextbufs[0][0] == '~')
+		{
+		defdupchar = findfiletype (&contextbufs[0][1], 1, (int *) NULL);
+		if (defdupchar < 0)
+		    defdupchar = 0;
+		}
+	    else
+		{
+		if (contextbufs[0][0] == '^')
+		    {
+		    /* Strip off leading uparrow */
+		    for (cp1 = contextbufs[0], cp2 = contextbufs[0] + 1;
+		      (*cp1++ = *cp2++) != '\0';
+		      )
+			;
+		    contextoffset++;
+		    }
+		checkline (stdout);
 		}
-	    checkline(stdout);
 	    }
+	if (hadnl)
+	    contextoffset = 0;
+	else
+	    contextoffset += bufsize;
 #ifndef USG
 	if (sflag)
 	    {
