@@ -1,6 +1,6 @@
 #ifndef lint
 static char Rcs_Id[] =
-    "$Id: defmt.c,v 1.36 1994/10/25 05:46:34 geoff Exp $";
+    "$Id: defmt.c,v 1.39 1995/01/08 23:23:54 geoff Exp $";
 #endif
 
 /*
@@ -54,6 +54,19 @@ static char Rcs_Id[] =
 
 /*
  * $Log: defmt.c,v $
+ * Revision 1.39  1995/01/08  23:23:54  geoff
+ * Fix typos in a couple of comments.
+ *
+ * Revision 1.38  1995/01/03  19:24:14  geoff
+ * Add code to handle the LaTeX \verb command.
+ *
+ * Revision 1.37  1994/12/27  23:08:54  geoff
+ * Fix a bug in TeX backslash processing that caused ispell to become
+ * confused when it encountered an optional argument to a
+ * double-backslash command.  Be a little smarter about scanning for
+ * curly-brace matches, so that we avoid missing a math-mode transition
+ * during the scan.
+ *
  * Revision 1.36  1994/10/25  05:46:34  geoff
  * Recognize a few more Latex commands: pagestyle, pagenumbering,
  * setcounter, addtocounter, setlength, addtolength, settowidth.
@@ -126,6 +139,8 @@ static int	TeX_strncmp P ((char * a, char * b, int n));
 			((c) == TEXDOLLAR) || \
 			((c) == TEXPERCENT))
 
+static int	    TeX_comment = 0;
+
 /*
  * The following variables are used to save the parsing state when
  * processing comments.  This allows comments to be parsed without
@@ -576,6 +591,11 @@ static int TeX_math_begin (bufp)
     while (**bufp == TEXBACKSLASH)
 	{
 	(*bufp)++; /* check for null char here? */
+	if (**bufp == TEXBACKSLASH)
+	    {
+	    (*bufp)++;
+	    continue;
+	    }
 	if (**bufp == TEXLEFTPAREN  ||  **bufp == TEXLEFTSQUARE)
 	    return 1;
 	if (TeX_strncmp (*bufp, "begin", 5) == 0)
@@ -753,14 +773,14 @@ static void TeX_skip_parens (bufp)
     char **	bufp;
     {
 
-    while (**bufp  &&  **bufp != TEXRIGHTCURLY)
+    while (**bufp  &&  **bufp != TEXRIGHTCURLY  &&  **bufp != TEXDOLLAR)
 	(*bufp)++;
     }
 
 static void TeX_open_paren (bufp)
     char **	bufp;
     {
-    while (**bufp  &&  **bufp != TEXLEFTCURLY)
+    while (**bufp  &&  **bufp != TEXLEFTCURLY  &&  **bufp != TEXDOLLAR)
 	(*bufp)++;
     }
 
@@ -768,6 +788,7 @@ static void TeX_skip_check (bufp)
     char **	bufp;
     {
     int		charlen;
+    int		skip_ch;
 
     /* ADDITIONALLY, MAY WANT TO ADD:
      * documentstyle, pagestyle, pagenumbering
@@ -819,13 +840,20 @@ static void TeX_skip_check (bufp)
 		LaTeX_Mode = 'r';
 	    }
 	}
+    else if (TeX_strncmp (*bufp, "verb", 4) == 0)
+	{
+	skip_ch = (*bufp)[4];
+	*bufp += 5;
+	while (**bufp != skip_ch  &&  **bufp != '\0')
+	    (*bufp)++;
+	}
     else
 	{
 	/* Optional tex arguments sometimes should and
 	** sometimes shouldn't be checked
 	** (eg \section [C Programming] {foo} vs
 	**     \rule [3em] {0.015in} {5em})
-	** SO -- we'll just igore it rather than make a
+	** SO -- we'll always spell-check it rather than make a
 	** full LaTeX parser.
 	*/
 
@@ -849,7 +877,7 @@ static void TeX_skip_check (bufp)
 
 /*
  * TeX_strncmp is like strncmp, except that it returns inequality if
- * the following character of a is not alphabetic.  We do not use
+ * the following character of a is alphabetic.  We do not use
  * iswordch here because TeX itself won't normally accept
  * nonalphabetics (except maybe on ISO Latin-1 installations?  I'll
  * have to look into that).  As a special hack, because LaTeX uses the
